==> lindorm-all-client-2.0.5-sources.jar: com/aliyun/lindorm/tsdb/client/Driver.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.aliyun.lindorm.tsdb.client;

import org.apache.calcite.avatica.BuiltInConnectionProperty;
import org.apache.calcite.avatica.DriverVersion;
import org.apache.calcite.avatica.remote.HADriver;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Properties;

public class Driver extends HADriver {

  public static final String CONNECT_STRING_PREFIX = "jdbc:lindorm:tsdb:";

  public final static String JDBC_PROTOCOL_TERMINATOR = ";";
  public final static String PROPERTY_SEPARATOR = "=";

  private static final String DEFAULT_SERIALIZATION = "PROTOBUF";
  private static final String DEFAULT_ENDPOINT = "/api/v2/avatica";

  static {
    new Driver().register();
  }

  public Driver() {
    super();
  }

  @Override
  protected DriverVersion createDriverVersion() {
    return DriverVersion.load(
        Driver.class,
        "com-aliyun-lindorm-tsdb-jdbc.properties",
        "Lindorm TSDB JDBC Driver",
        "unknown version",
        "Lindorm TSDB",
        "unknown version");
  }

  @Override
  protected String getConnectStringPrefix() {
    return CONNECT_STRING_PREFIX;
  }

  @Override
  public Connection connect(String url, Properties info) throws SQLException {
    if (!acceptsURL(url)) {
      return null;
    }

    if ((!info.containsKey("user")) || (!info.containsKey("password"))) {
      throw new SQLException("user and password can't be null");
    }

    String prefix = this.getConnectStringPrefix();
    String urlSuffix = url.substring(prefix.length());
    info.putAll(readExtraConfigFromUrl(urlSuffix));
    final String urlString = info.getProperty(BuiltInConnectionProperty.URL.camelName());
    if (urlString != null) {
      info.put(BuiltInConnectionProperty.URL.camelName(), urlString + DEFAULT_ENDPOINT);
    }
    info.put(BuiltInConnectionProperty.SERIALIZATION.camelName(), DEFAULT_SERIALIZATION);
    setConnectionInfo(info);
    Connection conn = super.connect(CONNECT_STRING_PREFIX, info);
    String schemaName = info.getProperty(BuiltInConnectionProperty.SCHEMA.camelName());
    if (schemaName != null) {
      conn.setSchema(schemaName);
    }
    return conn;
  }


  /**
   * @param url example jdbc:lindorm:tsdb:[key=value[;key=value...]]
   * @return
   */
  private Properties readExtraConfigFromUrl(String url) {
    Properties info = new Properties();
    String[] kvs = url.split(JDBC_PROTOCOL_TERMINATOR);
    for (String kv: kvs){
      String[] fields = kv.split(PROPERTY_SEPARATOR);
      if (fields.length == 2) {
        info.put(fields[0].trim(), fields[1].trim());
      }
    }
    return info;
  }
}

==> lindorm-all-client-2.0.9-sources.jar: com/aliyun/lindorm/tsdb/client/Driver.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.aliyun.lindorm.tsdb.client;

import org.apache.calcite.avatica.BuiltInConnectionProperty;
import org.apache.calcite.avatica.DriverVersion;
import org.apache.calcite.avatica.remote.AuthenticationType;
import org.apache.calcite.avatica.remote.HADriver;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Properties;

public class Driver extends HADriver {

  public static final String CONNECT_STRING_PREFIX = "jdbc:lindorm:tsdb:";

  public final static String JDBC_PROTOCOL_TERMINATOR = ";";
  public final static String PROPERTY_SEPARATOR = "=";

  private static final String DEFAULT_SERIALIZATION = "PROTOBUF";
  private static final String DEFAULT_ENDPOINT = "/api/v2/avatica";

  static {
    new Driver().register();
  }

  public Driver() {
    super();
  }

  @Override
  protected DriverVersion createDriverVersion() {
    return DriverVersion.load(
        Driver.class,
        "com-aliyun-lindorm-tsdb-jdbc.properties",
        "Lindorm TSDB JDBC Driver",
        "unknown version",
        "Lindorm TSDB",
        "unknown version");
  }

  @Override
  protected String getConnectStringPrefix() {
    return CONNECT_STRING_PREFIX;
  }

  @Override
  public Connection connect(String url, Properties info) throws SQLException {
    if (!acceptsURL(url)) {
      return null;
    }

    if (info.containsKey("user") && info.containsKey("password")) {
      info.put(BuiltInConnectionProperty.AUTHENTICATION.name(),
              AuthenticationType.BASIC.name());
      info.put(BuiltInConnectionProperty.AVATICA_USER.name(),
              info.getProperty("user"));
      info.put(BuiltInConnectionProperty.AVATICA_PASSWORD.name(),
              info.getProperty("password"));
    }

    String prefix = this.getConnectStringPrefix();
    String urlSuffix = url.substring(prefix.length());
    info.putAll(readExtraConfigFromUrl(urlSuffix));
    info.put(BuiltInConnectionProperty.TIME_ZONE.name(), "GMT");
    final String urlString = info.getProperty(BuiltInConnectionProperty.URL.camelName());
    if (urlString != null) {
      info.put(BuiltInConnectionProperty.URL.camelName(), urlString + DEFAULT_ENDPOINT);
    }
    info.put(BuiltInConnectionProperty.SERIALIZATION.camelName(), DEFAULT_SERIALIZATION);
    setConnectionInfo(info);
    Connection conn = super.connect(CONNECT_STRING_PREFIX, info);
    String schemaName = info.getProperty(BuiltInConnectionProperty.SCHEMA.camelName());
    if (schemaName != null) {
      conn.setSchema(schemaName);
    }
    return conn;
  }


  /**
   * @param url example jdbc:lindorm:tsdb:[key=value[;key=value...]]
   * @return
   */
  private Properties readExtraConfigFromUrl(String url) {
    Properties info = new Properties();
    String[] kvs = url.split(JDBC_PROTOCOL_TERMINATOR);
    for (String kv: kvs){
      String[] fields = kv.split(PROPERTY_SEPARATOR);
      if (fields.length == 2) {
        info.put(fields[0].trim(), fields[1].trim());
      }
    }
    return info;
  }
}
